@@ -1,13 +1,12 @@
 #include "mod/common/db/bib/db.h"
 
-#include <net/ip6_checksum.h>
 #include <linux/ktime.h>
+#include <net/ip6_checksum.h>
 
 #include "common/constants.h"
 #include "mod/common/icmp_wrapper.h"
 #include "mod/common/linux_version.h"
 #include "mod/common/log.h"
-#include "mod/common/route.h"
 #include "mod/common/wkmalloc.h"
 #include "mod/common/db/rbtree.h"
 #include "mod/common/db/bib/pkt_queue.h"
@@ -207,9 +206,10 @@ static void tbtobe(struct tabled_bib *tabled, struct bib_entry *bib)
 	if (!bib)
 		return;
 
-	bib->ipv6 = tabled->src6;
-	bib->ipv4 = tabled->src4;
+	bib->addr6 = tabled->src6;
+	bib->addr4 = tabled->src4;
 	bib->l4_proto = tabled->proto;
+	bib->is_static = tabled->is_static;
 }
 
 static unsigned long get_timeout(struct xlator *jool,
@@ -302,19 +302,19 @@ static struct bib_table *get_table(struct bib *db, l4_protocol proto)
 	return NULL;
 }
 
-static void kill_stored_pkt(struct bib_table *table,
+static void kill_stored_pkt(struct xlator *jool, struct bib_table *table,
 		struct tabled_session *session)
 {
 	if (!session->stored)
 		return;
 
-	log_debug("Deleting stored type 2 packet.");
+	__log_debug(jool, "Deleting stored type 2 packet.");
 	kfree_skb(session->stored);
 	session->stored = NULL;
 	table->pkt_count--;
 }
 
-int bib_setup(void)
+static int bib_setup(void)
 {
 	bib_cache = kmem_cache_create("bib_nodes",
 			sizeof(struct tabled_bib),
@@ -327,6 +327,7 @@ int bib_setup(void)
 			0, 0, NULL);
 	if (!session_cache) {
 		kmem_cache_destroy(bib_cache);
+		bib_cache = NULL;
 		return -ENOMEM;
 	}
 
@@ -335,8 +336,13 @@ int bib_setup(void)
 
 void bib_teardown(void)
 {
+	if (!bib_cache)
+		return;
+
 	kmem_cache_destroy(bib_cache);
+	bib_cache = NULL;
 	kmem_cache_destroy(session_cache);
+	session_cache = NULL;
 }
 
 static enum session_fate just_die(struct session_entry *session, void *arg)
@@ -366,7 +372,7 @@ static void init_table(struct bib_table *table,
 
 	init_expirer(&table->trans_timer, trans_timeout, SESSION_TIMER_TRANS,
 			just_die);
-	/* TODO "just_die"? what about the stored packet? */
+	/* TODO (warning) "just_die"? what about the stored packet? */
 	init_expirer(&table->syn4_timer, TCP_INCOMING_SYN, SESSION_TIMER_SYN4,
 			just_die);
 	table->pkt_count = 0;
@@ -376,24 +382,37 @@ static void init_table(struct bib_table *table,
 struct bib *bib_alloc(void)
 {
 	struct bib *db;
+	bool cache_created;
+
+	cache_created = false;
+	if (!bib_cache) {
+		if (bib_setup())
+			return NULL;
+		cache_created = true;
+	}
 
 	db = wkmalloc(struct bib, GFP_KERNEL);
 	if (!db)
-		return NULL;
+		goto db_alloc_fail;
 
 	init_table(&db->udp, UDP_DEFAULT, 0, just_die);
 	init_table(&db->tcp, TCP_EST, TCP_TRANS, tcp_est_expire_cb);
 	init_table(&db->icmp, ICMP_DEFAULT, 0, just_die);
 
 	db->tcp.pkt_queue = pktqueue_alloc();
-	if (!db->tcp.pkt_queue) {
-		wkfree(struct bib, db);
-		return NULL;
-	}
+	if (!db->tcp.pkt_queue)
+		goto pktqueue_alloc_fail;
 
 	kref_init(&db->refs);
 
 	return db;
+
+pktqueue_alloc_fail:
+	wkfree(struct bib, db);
+db_alloc_fail:
+	if (cache_created)
+		bib_teardown();
+	return NULL;
 }
 
 void bib_get(struct bib *db)
@@ -410,7 +429,7 @@ static void release_session(struct rb_node *node, void *arg)
 	struct tabled_session *session = node2session(node);
 
 	if (session->stored) {
-		icmp64_send(session->stored, ICMPERR_PORT_UNREACHABLE, 0);
+		icmp64_send(NULL, session->stored, ICMPERR_PORT_UNREACHABLE, 0);
 		kfree_skb(session->stored);
 	}
 
@@ -528,7 +547,8 @@ static void log_new_session(struct xlator *jool, struct tabled_session *session)
  * This function does not actually send the probe; it merely prepares it so the
  * caller can commit to sending it after releasing the spinlock.
  */
-static void handle_probe(struct bib_table *table,
+static void handle_probe(struct xlator *jool,
+		struct bib_table *table,
 		struct list_head *probes,
 		struct tabled_session *session,
 		struct session_entry *tmp)
@@ -569,7 +589,7 @@ discard_probe:
 	 * we do not want that massive thing to linger in the database anymore,
 	 * especially if we failed due to a memory allocation.
 	 */
-	kill_stored_pkt(table, session);
+	kill_stored_pkt(jool, table, session);
 }
 
 static void rm(struct xlator *jool,
@@ -581,7 +601,7 @@ static void rm(struct xlator *jool,
 	struct tabled_bib *bib = session->bib;
 
 	if (session->stored)
-		handle_probe(table, probes, session, tmp);
+		handle_probe(jool, table, probes, session, tmp);
 
 	rb_erase(&session->tree_hook, &bib->sessions);
 	list_del(&session->list_hook);
@@ -673,7 +693,7 @@ static bool decide_fate(struct xlator *jool,
 	session->state = tmp.state;
 	session->update_time = tmp.update_time;
 	if (!tmp.has_stored)
-		kill_stored_pkt(table, session);
+		kill_stored_pkt(jool, table, session);
 	/* Also the expirer, which is down below. */
 
 	switch (fate) {
@@ -682,8 +702,11 @@ static bool decide_fate(struct xlator *jool,
 		break;
 
 	case FATE_PROBE:
-		/* TODO ICMP errors aren't supposed to drop down to TRANS. */
-		handle_probe(table, probes, session, &tmp);
+		/*
+		 * TODO (warning) ICMP errors aren't supposed to drop down to
+		 * TRANS.
+		 */
+		handle_probe(jool, table, probes, session, &tmp);
 		/* Fall through. */
 	case FATE_TIMER_TRANS:
 		handle_fate_timer(session, &table->trans_timer);
@@ -721,12 +744,13 @@ static bool decide_fate(struct xlator *jool,
  *
  * Best if not called with spinlocks held.
  */
-static void send_probe_packet(struct net *ns, struct session_entry *session)
+static void send_probe_packet(struct xlator *jool, struct session_entry *session)
 {
-	struct packet pkt;
 	struct sk_buff *skb;
 	struct ipv6hdr *iph;
 	struct tcphdr *th;
+	struct flowi6 flow;
+	struct dst_entry *dst;
 	int error;
 
 	unsigned int l3_hdr_len = sizeof(*iph);
@@ -734,7 +758,7 @@ static void send_probe_packet(struct net *ns, struct session_entry *session)
 
 	skb = alloc_skb(LL_MAX_HEADER + l3_hdr_len + l4_hdr_len, GFP_ATOMIC);
 	if (!skb) {
-		log_debug("Could now allocate a probe packet.");
+		__log_debug(jool, "Could not allocate a probe packet.");
 		goto fail;
 	}
 
@@ -780,34 +804,43 @@ static void send_probe_packet(struct net *ns, struct session_entry *session)
 			IPPROTO_TCP, csum_partial(th, l4_hdr_len, 0));
 	skb->ip_summed = CHECKSUM_UNNECESSARY;
 
-	pkt_fill(&pkt, skb, L3PROTO_IPV6, L4PROTO_TCP, NULL, th + 1, NULL);
+	memset(&flow, 0, sizeof(flow));
+	flow.flowi6_scope = RT_SCOPE_UNIVERSE;
+	flow.flowi6_proto = NEXTHDR_TCP;
+	flow.saddr = iph->saddr;
+	flow.daddr = iph->daddr;
+	flow.fl6_sport = th->source;
+	flow.fl6_dport = th->dest;
 
-	if (!route6(ns, &pkt)) {
-		kfree_skb(skb);
-		goto fail;
-	}
+	dst = route6(jool, &flow);
+	if (!dst)
+		goto revert;
+
+	skb_dst_set(skb, dst);
 
 	/* Implicit kfree_skb(skb) here. */
 #if LINUX_VERSION_AT_LEAST(4, 4, 0, 8, 0)
-	error = dst_output(ns, NULL, skb);
+	error = dst_output(jool->ns, NULL, skb);
 #else
 	error = dst_output(skb);
 #endif
 	if (error) {
-		log_debug("dst_output() returned errcode %d.", error);
+		__log_debug(jool, "dst_output() returned errcode %d.", error);
 		goto fail;
 	}
 
 	return;
 
+revert:
+	kfree_skb(skb);
 fail:
-	log_debug("A TCP connection will probably break.");
+	__log_debug(jool, "A TCP connection will probably break.");
 }
 
 /**
  * Sends all the probes and ICMP errors listed in @probes.
  */
-static void post_fate(struct net *ns, struct list_head *probes)
+static void post_fate(struct xlator *jool, struct list_head *probes)
 {
 	struct probing_session *probe;
 	struct probing_session *tmp;
@@ -815,11 +848,12 @@ static void post_fate(struct net *ns, struct list_head *probes)
 	list_for_each_entry_safe(probe, tmp, probes, list_hook) {
 		if (probe->skb) {
 			/* The "probe" is not a probe; it's an ICMP error. */
-			icmp64_send(probe->skb, ICMPERR_PORT_UNREACHABLE, 0);
+			icmp64_send(jool, probe->skb, ICMPERR_PORT_UNREACHABLE,
+					0);
 			kfree_skb(probe->skb);
 		} else {
 			/* Actual TCP probe. */
-			send_probe_packet(ns, &probe->session);
+			send_probe_packet(jool, &probe->session);
 		}
 		wkfree(struct probing_session, probe);
 	}
@@ -1351,18 +1385,18 @@ static int upgrade_pktqueue_session(struct xlator *jool,
 		 * joold anyway. And this is only a problem in active-active
 		 * scenarios.
 		 */
-		pktqueue_put_node(sos);
+		pktqueue_put_node(jool, sos);
 		return -ESRCH;
 	}
 
-	log_debug("Simultaneous Open!");
+	__log_debug(jool, "Simultaneous Open!");
 	/*
 	 * We're going to pretend that @sos has been a valid V4 INIT session all
 	 * along.
 	 */
 	error = alloc_bib_session(old);
 	if (error) {
-		pktqueue_put_node(sos);
+		pktqueue_put_node(jool, sos);
 		return error;
 	}
 
@@ -1401,14 +1435,14 @@ static int upgrade_pktqueue_session(struct xlator *jool,
 	attach_timer(session, &table->syn4_timer);
 	jstat_inc(jool->stats, JSTAT_SESSIONS);
 
-	pktqueue_put_node(sos);
+	pktqueue_put_node(jool, sos);
 
 	log_new_bib(jool, bib);
 	log_new_session(jool, session);
 	return 0;
 
 trainwreck:
-	pktqueue_put_node(sos);
+	pktqueue_put_node(jool, sos);
 	free_bib(bib);
 	free_session(session);
 	return -EINVAL;
@@ -1481,7 +1515,7 @@ static int find_bib_session6(struct xlator *jool,
 		 * BIB entry and recompute it from scratch.
 		 * https://github.com/NICMx/Jool/issues/216
 		 */
-		log_debug("Issue #216.");
+		__log_debug(jool, "Issue #216.");
 		detach_bib(jool, table, old->bib);
 		add_to_delete_list(bdl, &old->bib->hook4);
 
@@ -1750,7 +1784,7 @@ verdict bib_add_tcp6(struct xlation *state,
 			tbtobs(old.bib, &state->entries);
 			result = VERDICT_CONTINUE;
 		} else {
-			log_debug("Packet is not SYN and lacks state.");
+			log_debug(state, "Packet is not SYN and lacks state.");
 			result = drop(state, JSTAT_SYN6_EXPECTED);
 		}
 		goto end;
@@ -1822,14 +1856,14 @@ verdict bib_add_tcp4(struct xlation *state,
 			tbtobs(old.bib, &state->entries);
 			result = VERDICT_CONTINUE;
 		} else {
-			log_debug("Packet is not SYN and lacks state.");
+			log_debug(state, "Packet is not SYN and lacks state.");
 			result = drop(state, JSTAT_SYN4_EXPECTED);
 		}
 		goto end;
 	}
 
 	if (GLOBALS(state).drop_external_tcp) {
-		log_debug("Externally initiated TCP connections are prohibited.");
+		log_debug(state, "Externally initiated TCP connections are prohibited.");
 		result = drop(state, JSTAT_V4_SYN);
 		goto end;
 	}
@@ -1837,7 +1871,7 @@ verdict bib_add_tcp4(struct xlation *state,
 	if (!old.bib) {
 		bool too_many;
 
-		log_debug("Potential Simultaneous Open; storing type 1 packet.");
+		log_debug(state, "Potential Simultaneous Open; storing type 1 packet.");
 		too_many = table->pkt_count >= GLOBALS(state).max_stored_pkts;
 		error = pktqueue_add(table->pkt_queue, pkt, dst6, too_many);
 		switch (error) {
@@ -1846,7 +1880,7 @@ verdict bib_add_tcp4(struct xlation *state,
 			table->pkt_count++;
 			goto end;
 		case -EEXIST:
-			log_debug("Simultaneous Open already exists.");
+			log_debug(state, "Simultaneous Open already exists.");
 			result = drop(state, JSTAT_SO_EXISTS);
 			goto end;
 		case -ENOSPC:
@@ -1867,7 +1901,7 @@ verdict bib_add_tcp4(struct xlation *state,
 		if (table->pkt_count >= GLOBALS(state).max_stored_pkts)
 			goto too_many_pkts;
 
-		log_debug("Potential Simultaneous Open; storing type 2 packet.");
+		log_debug(state, "Potential Simultaneous Open; storing type 2 packet.");
 		new->stored = pkt_original_pkt(pkt)->skb;
 		result = stolen(state, JSTAT_TYPE2PKT);
 		table->pkt_count++;
@@ -1893,7 +1927,7 @@ end:
 too_many_pkts:
 	spin_unlock_bh(&table->lock);
 	free_session(new);
-	log_debug("Too many Simultaneous Opens.");
+	log_debug(state, "Too many Simultaneous Opens.");
 	/* Fall back to assume there's no SO. */
 	return drop_icmp(state, JSTAT_SO_FULL, ICMPERR_PORT_UNREACHABLE, 0);
 }
@@ -1919,8 +1953,8 @@ int bib_find(struct bib *db, struct tuple *tuple, struct bib_session *result)
 		return error;
 
 	result->bib_set = true;
-	result->session.src6 = tmp.ipv6;
-	result->session.src4 = tmp.ipv4;
+	result->session.src6 = tmp.addr6;
+	result->session.src4 = tmp.addr4;
 	result->session.proto = tmp.l4_proto;
 	return 0;
 }
@@ -2011,7 +2045,7 @@ static void clean_table(struct xlator *jool, struct bib_table *table)
 	}
 	spin_unlock_bh(&table->lock);
 
-	post_fate(jool->ns, &probes);
+	post_fate(jool, &probes);
 	pktqueue_clean(&icmps);
 }
 
@@ -2057,7 +2091,7 @@ static struct rb_node *find_starting_point(struct bib_table *table,
 }
 
 int bib_foreach(struct bib *db, l4_protocol proto,
-		struct bib_foreach_func *func,
+		bib_foreach_entry_cb cb, void *cb_arg,
 		const struct ipv4_transport_addr *offset)
 {
 	struct bib_table *table;
@@ -2076,7 +2110,7 @@ int bib_foreach(struct bib *db, l4_protocol proto,
 	for (; node && !error; node = rb_next(node)) {
 		tabled = bib4_entry(node);
 		tbtobe(tabled, &bib);
-		error = func->cb(&bib, tabled->is_static, func->arg);
+		error = cb(&bib, cb_arg);
 	}
 
 	spin_unlock_bh(&table->lock);
@@ -2161,7 +2195,7 @@ static void find_session_offset(struct bib_table *table,
 				node = node2session(rb_next(&node->tree_hook)))
 
 int bib_foreach_session(struct xlator *jool, l4_protocol proto,
-		struct session_foreach_func *func,
+		session_foreach_entry_cb cb, void *cb_arg,
 		struct session_foreach_offset *offset)
 {
 	struct bib_table *table;
@@ -2188,7 +2222,7 @@ int bib_foreach_session(struct xlator *jool, l4_protocol proto,
 	foreach_bib(table, pos.bib) {
 goto_bib:	foreach_session(&pos.bib->sessions, pos.session) {
 goto_session:		tstose(jool, pos.session, &tmp);
-			error = func->cb(&tmp, func->arg);
+			error = cb(&tmp, cb_arg);
 			if (error)
 				goto end;
 		}
@@ -2244,8 +2278,8 @@ int bib_find4(struct bib *db, l4_protocol proto,
 
 static void bib2tabled(struct bib_entry *bib, struct tabled_bib *tabled)
 {
-	tabled->src6 = bib->ipv6;
-	tabled->src4 = bib->ipv4;
+	tabled->src6 = bib->addr6;
+	tabled->src4 = bib->addr4;
 	tabled->proto = bib->l4_proto;
 	tabled->is_static = true;
 	tabled->sessions = RB_ROOT;
@@ -2260,6 +2294,10 @@ static int __bib_add_static(struct xlator *jool, struct bib_entry *new,
 	struct tree_slot slot6;
 	struct tree_slot slot4;
 
+	__log_debug(jool, "Adding static BIB entry (%pI6c#%u, %pI4#%u).",
+			&new->addr6.l3, new->addr6.l4,
+			&new->addr4.l3, new->addr4.l4);
+
 	table = get_table(jool->nat64.bib, new->l4_proto);
 	if (!table)
 		return -EINVAL;
@@ -2293,7 +2331,7 @@ static int __bib_add_static(struct xlator *jool, struct bib_entry *new,
 	 * going to retry anyway, so let's just forget the packets instead.
 	 */
 	if (new->l4_proto == L4PROTO_TCP)
-		pktqueue_rm(jool->nat64.bib->tcp.pkt_queue, &new->ipv4);
+		pktqueue_rm(jool->nat64.bib->tcp.pkt_queue, &new->addr4);
 
 	spin_unlock_bh(&table->lock);
 	return 0;
@@ -2323,10 +2361,10 @@ int bib_add_static(struct xlator *jool, struct bib_entry *new)
 		break;
 	case -EEXIST:
 		log_err("Entry %pI4#%u|%pI6c#%u collides with %pI4#%u|%pI6c#%u.",
-				&new->ipv4.l3, new->ipv4.l4,
-				&new->ipv6.l3, new->ipv6.l4,
-				&old.ipv4.l3, old.ipv4.l4,
-				&old.ipv6.l3, old.ipv6.l4);
+				&new->addr4.l3, new->addr4.l4,
+				&new->addr6.l3, new->addr6.l4,
+				&old.addr4.l3, old.addr4.l4,
+				&old.addr6.l3, old.addr6.l4);
 		break;
 	default:
 		log_err("Unknown error code: %d", error);
@@ -2474,10 +2512,10 @@ static void print_bib(struct rb_node *node, int tabs)
 
 void bib_print(struct bib *db)
 {
-	log_debug("TCP:");
+	LOG_DEBUG("TCP:");
 	print_bib(db->tcp.tree4.rb_node, 1);
-	log_debug("UDP:");
+	LOG_DEBUG("UDP:");
 	print_bib(db->udp.tree4.rb_node, 1);
-	log_debug("ICMP:");
+	LOG_DEBUG("ICMP:");
 	print_bib(db->icmp.tree4.rb_node, 1);
 }
