@@ -27,8 +27,7 @@ bool eamt_is_empty(struct eam_table *eamt);
 
 /* Do-not-use-when-you-can't-sleep-functions */
 
-int eamt_add(struct eam_table *eamt, struct ipv6_prefix *prefix6,
-		struct ipv4_prefix *prefix4, bool force);
+int eamt_add(struct eam_table *jool, struct eamt_entry *new, bool force);
 int eamt_rm(struct eam_table *eamt, struct ipv6_prefix *prefix6,
 		struct ipv4_prefix *prefix4);
 void eamt_flush(struct eam_table *eamt);
