@@ -20,7 +20,7 @@ int handle_eamt_foreach(struct sk_buff *skb, struct genl_info *info)
 	struct ipv4_prefix offset, *offset_ptr;
 	int error;
 
-	error = request_handle_start(info, XT_SIIT, &jool);
+	error = request_handle_start(info, XT_SIIT, &jool, true);
 	if (error)
 		return jresponse_send_simple(NULL, info, error);
 
@@ -69,7 +69,7 @@ int handle_eamt_add(struct sk_buff *skb, struct genl_info *info)
 	struct eamt_entry addend;
 	int error;
 
-	error = request_handle_start(info, XT_SIIT, &jool);
+	error = request_handle_start(info, XT_SIIT, &jool, true);
 	if (error)
 		return jresponse_send_simple(NULL, info, error);
 
@@ -95,7 +95,7 @@ int handle_eamt_rm(struct sk_buff *skb, struct genl_info *info)
 	struct ipv4_prefix prefix4, *prefix4_ptr;
 	int error;
 
-	error = request_handle_start(info, XT_SIIT, &jool);
+	error = request_handle_start(info, XT_SIIT, &jool, true);
 	if (error)
 		return jresponse_send_simple(NULL, info, error);
 
@@ -142,7 +142,7 @@ int handle_eamt_flush(struct sk_buff *skb, struct genl_info *info)
 	struct xlator jool;
 	int error;
 
-	error = request_handle_start(info, XT_SIIT, &jool);
+	error = request_handle_start(info, XT_SIIT, &jool, true);
 	if (error)
 		return jresponse_send_simple(NULL, info, error);
 
