@@ -17,11 +17,10 @@
  */
 struct pkt_metadata {
 	/*
+	 * Offset is from skb->data. Zero if there's no fragment header.
 	 * Note: The fact that a packet has a fragment header does not imply
 	 * that it is fragmented.
 	 */
-	bool has_frag_hdr;
-	/* Offset is from skb->data. Do not use if has_frag_hdr is false. */
 	unsigned int frag_offset;
 	/* Actual packet protocol; not tuple protocol. */
 	enum l4_protocol l4_proto;
@@ -47,27 +46,22 @@ static bool has_inner_pkt6(__u8 icmp6_type)
 /* It seems that this should never trigger ICMP errors. */
 static verdict truncated(struct xlation *state, const char *what)
 {
-	log_debug("The %s seems truncated.", what);
+	log_debug(state, "The %s seems truncated.", what);
 	return drop(state, JSTAT_SKB_TRUNCATED);
 }
 
 static verdict inhdr6(struct xlation *state, const char *msg)
 {
-	log_debug("%s", msg);
+	log_debug(state, "%s", msg);
 	return drop(state, JSTAT_HDR6);
 }
 
 static verdict inhdr4(struct xlation *state, const char *msg)
 {
-	log_debug("%s", msg);
+	log_debug(state, "%s", msg);
 	return drop(state, JSTAT_HDR4);
 }
 
-static void *skb_offset_to_ptr(struct sk_buff *skb, unsigned int offset)
-{
-	return ((void *) skb->data) + offset;
-}
-
 /**
  * Apparently, as of 2007, Netfilter modules can assume they are the sole owners
  * of their skbs (http://lists.openwall.net/netdev/2007/10/14/13).
@@ -167,7 +161,7 @@ static verdict summarize_skb6(struct xlation *state,
 	nexthdr = *ptr.nexthdr;
 	offset = hdr6_offset + sizeof(struct ipv6hdr);
 
-	meta->has_frag_hdr = false;
+	meta->frag_offset = 0;
 
 	do {
 		switch (nexthdr) {
@@ -206,7 +200,6 @@ static verdict summarize_skb6(struct xlation *state,
 			if (!ptr.frag)
 				return truncated(state, "fragment header");
 
-			meta->has_frag_hdr = true;
 			meta->frag_offset = offset;
 			is_first = is_first_frag6(ptr.frag);
 
@@ -217,8 +210,8 @@ static verdict summarize_skb6(struct xlation *state,
 		case NEXTHDR_HOP:
 		case NEXTHDR_ROUTING:
 		case NEXTHDR_DEST:
-			if (meta->has_frag_hdr) {
-				log_debug("There's a known extension header (%u) after Fragment.",
+			if (meta->frag_offset) {
+				log_debug(state, "There's a known extension header (%u) after Fragment.",
 						nexthdr);
 				return drop_icmp(state, JSTAT64_FRAG_THEN_EXT,
 						ICMPERR_FILTER, 0);
@@ -271,7 +264,7 @@ static verdict validate_inner6(struct xlation *state,
 	if (result != VERDICT_CONTINUE)
 		return result;
 
-	if (meta.has_frag_hdr) {
+	if (meta.frag_offset) {
 		ptr.frag = skb_hdr_ptr(state->in.skb, meta.frag_offset,
 				buffer.frag);
 		if (!ptr.frag)
@@ -290,7 +283,7 @@ static verdict validate_inner6(struct xlation *state,
 	}
 
 	if (!pskb_may_pull(state->in.skb, meta.payload_offset)) {
-		log_debug("Could not 'pull' the headers out of the skb.");
+		log_debug(state, "Could not 'pull' the headers out of the skb.");
 		return truncated(state, "inner headers");
 	}
 
@@ -320,13 +313,13 @@ static verdict handle_icmp6(struct xlation *state, struct pkt_metadata const *me
 	}
 
 	if (xlation_is_siit(state)
-			&& meta->has_frag_hdr
+			&& meta->frag_offset
 			&& is_icmp6_info(ptr.icmp->icmp6_type)) {
 		ptr.frag = skb_hdr_ptr(state->in.skb, meta->frag_offset, buffer.frag);
 		if (!ptr.frag)
 			return truncated(state, "fragment header");
 		if (is_fragmented_ipv6(ptr.frag)) {
-			log_debug("Packet is a fragmented ping; its checksum cannot be translated.");
+			log_debug(state, "Packet is a fragmented ping; its checksum cannot be translated.");
 			return drop(state, JSTAT_FRAGMENTED_PING);
 		}
 	}
@@ -355,7 +348,7 @@ verdict pkt_init_ipv6(struct xlation *state, struct sk_buff *skb)
 	if (result != VERDICT_CONTINUE)
 		return result;
 
-	log_debug("Packet addresses: %pI6c->%pI6c",
+	log_debug(state, "Packet: %pI6c->%pI6c",
 			&ipv6_hdr(skb)->saddr,
 			&ipv6_hdr(skb)->daddr);
 
@@ -379,12 +372,9 @@ verdict pkt_init_ipv6(struct xlation *state, struct sk_buff *skb)
 	state->in.l3_proto = L3PROTO_IPV6;
 	state->in.l4_proto = meta.l4_proto;
 	state->in.is_inner = 0;
-	state->in.is_hairpin = false;
-	state->in.hdr_frag = meta.has_frag_hdr
-			? skb_offset_to_ptr(skb, meta.frag_offset)
-			: NULL;
+	state->in.frag_offset = meta.frag_offset;
 	skb_set_transport_header(skb, meta.l4_offset);
-	state->in.payload = skb_offset_to_ptr(skb, meta.payload_offset);
+	state->in.payload_offset = meta.payload_offset;
 	state->in.original_pkt = &state->in;
 
 	return VERDICT_CONTINUE;
@@ -458,7 +448,7 @@ static verdict handle_icmp4(struct xlation *state, struct pkt_metadata *meta)
 	if (xlation_is_siit(state)
 			&& is_icmp4_info(ptr->type)
 			&& is_fragmented_ipv4(ip_hdr(state->in.skb))) {
-		log_debug("Packet is a fragmented ping; its checksum cannot be translated.");
+		log_debug(state, "Packet is a fragmented ping; its checksum cannot be translated.");
 		return drop(state, JSTAT_FRAGMENTED_PING);
 	}
 
@@ -473,7 +463,7 @@ static verdict summarize_skb4(struct xlation *state, struct pkt_metadata *meta)
 	hdr4 = ip_hdr(state->in.skb);
 	offset = skb_network_offset(state->in.skb) + (hdr4->ihl << 2);
 
-	meta->has_frag_hdr = false;
+	meta->frag_offset = 0;
 	meta->l4_offset = offset;
 	meta->payload_offset = offset;
 
@@ -527,7 +517,7 @@ verdict pkt_init_ipv4(struct xlation *state, struct sk_buff *skb)
 	if (result != VERDICT_CONTINUE)
 		return result;
 
-	log_debug("Packet addresses: %pI4->%pI4",
+	log_debug(state, "Packet: %pI4->%pI4",
 			&ip_hdr(skb)->saddr,
 			&ip_hdr(skb)->daddr);
 
@@ -536,17 +526,16 @@ verdict pkt_init_ipv4(struct xlation *state, struct sk_buff *skb)
 		return result;
 
 	if (!pskb_may_pull(skb, meta.payload_offset)) {
-		log_debug("Could not 'pull' the headers out of the skb.");
+		log_debug(state, "Could not 'pull' the headers out of the skb.");
 		return truncated(state, "headers");
 	}
 
 	state->in.l3_proto = L3PROTO_IPV4;
 	state->in.l4_proto = meta.l4_proto;
 	state->in.is_inner = false;
-	state->in.is_hairpin = false;
-	state->in.hdr_frag = NULL;
+	state->in.frag_offset = 0;
 	skb_set_transport_header(skb, meta.l4_offset);
-	state->in.payload = skb_offset_to_ptr(skb, meta.payload_offset);
+	state->in.payload_offset = meta.payload_offset;
 	state->in.original_pkt = &state->in;
 
 	return VERDICT_CONTINUE;
@@ -567,6 +556,8 @@ verdict pkt_init_ipv4(struct xlation *state, struct sk_buff *skb)
  * usual.
  *
  * Never use skb_pull() directly.
+ *
+ * TODO (fine) the 7915 code is breaking that rule.
  */
 unsigned char *jskb_pull(struct sk_buff *skb, unsigned int len)
 {
@@ -576,17 +567,6 @@ unsigned char *jskb_pull(struct sk_buff *skb, unsigned int len)
 	return result;
 }
 
-/**
- * skb_push() cannot return NULL at present, but maybe someday will.
- */
-unsigned char *jskb_push(struct sk_buff *skb, unsigned int len)
-{
-	unsigned char *result = skb_push(skb, len);
-	WARN(!result, "Bug: skb_push() returned NULL (skblen:%u len:%u).",
-			skb->len, len);
-	return result;
-}
-
 #define SIMPLE_MIN(a, b) ((a < b) ? a : b)
 
 void snapshot_record(struct pkt_snapshot *shot, struct sk_buff *skb)
